import subprocess
from .utils import *


def main():
    def check_spec() -> None:
        spec = fast_read(p('buildozer.spec'))
        lines = spec.split('\n')
        for line_num in range(len(lines)):
            line = lines[line_num]
            low = line.lower().replace('#', '').strip()
            if low.startswith('source.include_exts'):
                lines[line_num] = 'source.include_exts = py,png,jpg,kv,atlas,ttf,jpg,ico,ogg,mp3'
                continue
            if low.startswith('source.exclude_dirs'):
                lines[line_num] = 'source.exclude_dirs = tests, bin, __pycache__, .idea, dist, build'
                continue
            if low.startswith('requirements'):
                lines[line_num] = 'requirements = python3,pygame,Pillow,gsdl'
                continue
            if low.startswith('orientation'):
                lines[line_num] = 'orientation = landscape'
                continue
            if low.startswith('android.accept_sdk_license'):
                lines[line_num] = 'android.accept_sdk_license = True'
                continue
            if low.startswith('android.permissions'):
                lines[line_num] = 'android.permissions = INTERNET, READ_EXTERNAL_STORAGE, WRITE_EXTERNAL_STORAGE'
                continue
        fast_write(p('buildozer.spec'), '\n'.join(lines))

    for x in sys.argv[1:]:
        cur = x.lower().strip()
        if cur == '--help':
            print('\n\nArgs:')
            print('--init')
            print('--help')
            print('--build-debug')
            print('--build-release')
            sys.exit(0)
        if cur == '--init':
            subprocess.call([
                sys.executable,
                '-m',
                'buildozer',
                'init'
            ])
            check_spec()
            sys.exit(0)
        if cur == '--build-debug':
            subprocess.call([
                sys.executable,
                '-m',
                'buildozer',
                'android',
                'debug',
                'deploy',
                'run'
            ])
            sys.exit(0)
        if cur == '--build-release':
            subprocess.call([
                sys.executable,
                '-m',
                'buildozer',
                'android',
                'debug',
                'deploy',
                'run'
            ])
            sys.exit(0)
        print(f'Unknown arg: {cur}')
        sys.exit(1)


if __name__ == '__main__':
    main()



from .utils import *
from .frame import GFrame
import pygame


class GRect:
    def __init__(
            self,
            parent: GFrame,
            x: int = 0,
            y: int = 0,
            w: int = 0,
            h: int = 0,
            color: tuple = (0, 0, 0),
            width: int = 0,
            border_radius: int = -1,
            border_top_left_radius: int = -1,
            border_top_right_radius: int = -1,
            border_bottom_left_radius: int = -1,
            border_bottom_right_radius: int = -1,
            z_order: int = 0
    ):
        super(GRect, self).__init__()
        self.parent = parent
        self.surface = parent.surface
        self.x = x
        self.y = y
        self.w = w
        self.h = h
        self.catch_keyboard = False
        self.catch_mouse = True
        self.is_colliding = False
        self.is_visible = True
        self.is_enabled = True
        self.color = color
        self.width = width
        self.is_pressed = False
        self.border_radius = border_radius
        self.border_top_left_radius = border_top_left_radius
        self.border_top_right_radius = border_top_right_radius
        self.border_bottom_left_radius = border_bottom_left_radius
        self.border_bottom_right_radius = border_bottom_right_radius
        self.z_order = z_order
        self.last_btn = 0
        self.parent.add_child(self)

    def draw(self, delta: int, parent: GFrame):
        pygame.draw.rect(
            self.surface,
            self.color,
            (self.x, self.y, self.w, self.h),
            self.width,
            self.border_radius,
            self.border_top_left_radius,
            self.border_top_right_radius,
            self.border_bottom_left_radius,
            self.border_bottom_right_radius
        )

    def on_mouse_move(self, x: int, y: int) -> None:
        pass

    def on_mouse_down(self, x: int, y: int, button: int = 0) -> None:
        pass

    def on_mouse_up(self, x: int, y: int, button: int = 0) -> None:
        pass

    def on_mouse_leave(self, x: int, y: int) -> None:
        pass

    def on_mouse_enter(self):
        pass

    def on_click(self, x, y):
        pass

    def on_mouse_move_(self, x: int, y: int) -> None:
        if self.is_pressed and self.last_btn == 1:
            if not is_object_colliding(self, x, y):
                self.is_colliding = False
                self.on_mouse_leave(x, y)
                return self.on_mouse_move(x, y)
        if not self.is_colliding:
            self.is_colliding = True
            self.on_mouse_enter()
        self.on_mouse_move(x, y)

    def on_mouse_leave_(self, x: int, y: int) -> None:
        self.is_colliding = False
        self.on_mouse_leave(x, y)

    def on_mouse_down_(self, x: int, y: int, button: int = 0) -> None:
        self.on_mouse_down(x, y, button)

    def on_mouse_up_(self, x: int, y: int, button: int = 0) -> None:
        self.on_mouse_up(x, y, button)
        if self.last_btn == button == 1 and self.is_colliding:
            self.on_click(x, y)



from .utils import *
from .frame import GFrame
from .label import GLabel
import pygame


class GWinButton:  # TODO: add GAnimatedWinButton
    def __init__(
            self,
            parent: GFrame,
            text: str,
            font: Union[pygame.font.Font, str],
            font_size: int = 0,
            text_color: tuple = (0, 0, 0),
            anti_aliasing: bool = True,
            x: int = 0,
            y: int = 0,
            w: int = 0,
            h: int = 0,
            border_radius: int = -1,
            border_top_left_radius: int = -1,
            border_top_right_radius: int = -1,
            border_bottom_left_radius: int = -1,
            border_bottom_right_radius: int = -1,
            z_order: int = 0
    ):
        super(GWinButton, self).__init__()
        self.parent = parent
        self.surface = parent.surface
        self.surf = pygame.Surface((w, h), pygame.SRCALPHA)
        self.x = x
        self.y = y
        self.w = w
        self.h = h
        self.catch_keyboard = False
        self.catch_mouse = True
        self.is_visible = True
        self.is_enabled = True
        self.last_btn = 0
        self.is_pressed = False
        self.z_order = z_order
        self.color = (225, 225, 225)
        self.hover_color = (229, 241, 251)
        self.click_color = (205, 228, 247)
        self.border_color = (173, 173, 173)
        self.border_hover_color = (0, 120, 215)
        self.border_click_color = (1, 85, 154)
        self.disabled_color = (204, 204, 204)
        self.disabled_hover_color = (191, 191, 191)
        self.border_radius = border_radius
        self.border_top_left_radius = border_top_left_radius
        self.border_top_right_radius = border_top_right_radius
        self.border_bottom_left_radius = border_bottom_left_radius
        self.border_bottom_right_radius = border_bottom_right_radius
        self.label = GLabel(
            None,
            text,
            font,
            font_size,
            text_color,
            auto_size=True,
            stretch=False,
            anti_aliasing=anti_aliasing
        )
        self.text_w, self.text_h = self.label.orig_image.get_size()
        self.label_pos = (
            round(self.w / 2 - self.text_w / 2),
            round(self.h / 2 - self.text_h / 2)
        )
        self.is_colliding = False
        self.parent.add_child(self)

    def update(self):
        self.text_w, self.text_h = self.label.orig_image.get_size()
        self.label_pos = (
            round(self.w / 2 - self.text_w / 2),
            round(self.h / 2 - self.text_h / 2)
        )

    def draw(self, delta: int, parent: GFrame):
        self.surf.fill((0, 0, 0, 0))
        if not self.is_enabled:
            pygame.draw.rect(
                self.surf,
                self.disabled_color,
                (0, 0, self.w, self.h),
                0,
                self.border_radius,
                self.border_top_left_radius,
                self.border_top_right_radius,
                self.border_bottom_left_radius,
                self.border_bottom_right_radius
            )
            pygame.draw.rect(
                self.surf,
                self.disabled_hover_color,
                (0, 0, self.w, self.h),
                1,
                self.border_radius,
                self.border_top_left_radius,
                self.border_top_right_radius,
                self.border_bottom_left_radius,
                self.border_bottom_right_radius
            )
            self.surf.blit(self.label.orig_image, self.label_pos)
            self.surface.blit(self.surf, (self.x + self.parent.scroll_x, self.y + self.parent.scroll_y))
            return
        col, b_col = (self.click_color, self.border_click_color) if self.is_pressed and self.last_btn == 1 else ((self.hover_color, self.border_hover_color) if self.is_colliding else (self.color, self.border_color))
        pygame.draw.rect(
            self.surf,
            col,
            (0, 0, self.w, self.h),
            0,
            self.border_radius,
            self.border_top_left_radius,
            self.border_top_right_radius,
            self.border_bottom_left_radius,
            self.border_bottom_right_radius
        )
        pygame.draw.rect(
            self.surf,
            b_col,
            (0, 0, self.w, self.h),
            1,
            self.border_radius,
            self.border_top_left_radius,
            self.border_top_right_radius,
            self.border_bottom_left_radius,
            self.border_bottom_right_radius
        )
        self.surf.blit(self.label.orig_image, self.label_pos)
        self.surface.blit(self.surf, (self.x + self.parent.scroll_x, self.y + self.parent.scroll_y))

    def on_mouse_move(self, x: int, y: int) -> None:
        pass

    def on_mouse_leave(self, x: int, y: int) -> None:
        pass

    def on_mouse_down(self, x: int, y: int, button: int = 0) -> None:
        pass

    def on_mouse_enter(self):
        pass

    def on_mouse_up(self, x: int, y: int, button: int = 0) -> None:
        pass

    def on_click(self, x, y):
        pass

    def on_mouse_move_(self, x: int, y: int) -> None:
        if self.is_pressed and self.last_btn == 1:
            if not is_object_colliding(self, x, y):
                self.is_colliding = False
                return self.on_mouse_move(x, y)
        if not self.is_colliding:
            self.is_colliding = True
            self.on_mouse_enter()
        self.on_mouse_move(x, y)

    def on_mouse_leave_(self, x: int, y: int) -> None:
        self.is_colliding = False
        self.on_mouse_leave(x, y)

    def on_mouse_down_(self, x: int, y: int, button: int = 0) -> None:
        self.on_mouse_down(x, y, button)

    def on_mouse_up_(self, x: int, y: int, button: int = 0) -> None:
        self.on_mouse_up(x, y, button)
        if self.last_btn == button == 1 and self.is_colliding:
            self.on_click(x, y)



from .utils import *
import pygame


class GFrame:
    def __init__(
            self,
            surface: pygame.Surface = None,
            x: int = 0,
            y: int = 0,
            w: int = 0,
            h: int = 0,
            is_main_window: bool = False,
            flags: int = 0,
            depth: int = None,
            display: int = None,
            vsync: int = None,
            z_order: int = 0
    ):
        super(GFrame, self).__init__()
        self.surface = surface if surface else pygame.Surface((w, h), flags, depth)
        self.x = x
        self.y = y
        self.w = w if w else surface.get_size()[0]
        self.h = h if h else surface.get_size()[1]
        self.is_main_window = is_main_window
        self.scroll_x = 0
        self.scroll_y = 0
        self.flags = flags if flags else self.surface.get_flags()
        self.depth = depth
        self.display = display
        self.vsync = vsync
        self.bg_color = (0, 0, 0)
        self.is_visible = True
        self.is_pressed = False
        self.is_enabled = True
        self.catch_mouse = True
        self.catch_keyboard = True
        self.pressed_child = None
        self.is_global_pressed = False
        self.child = []
        self.z_order = z_order
        self.last_hover = None
        self.wm_info = pygame.display.get_wm_info()
        self.hwnd = self.wm_info.get('window') or 0
        self.hdc = self.wm_info.get('hdc') or 0
        self.hinstance = self.wm_info.get('hinstance') or 0

    def on_before_update(self) -> None:
        pass

    def on_after_update(self) -> None:
        pass

    def on_resize(self, w: int, h: int) -> None:
        pass

    def on_quit(self) -> None:
        pass

    def on_global_mouse_move(self, x: int, y: int) -> None:
        pass

    def on_global_mouse_down(self, x: int, y: int, button: int) -> None:
        pass

    def on_global_mouse_up(self, x: int, y: int, button: int) -> None:
        pass

    def on_mouse_move(self, x: int, y: int) -> None:
        pass

    def on_mouse_down(self, x: int, y: int, button: int) -> None:
        pass

    def on_mouse_up(self, x: int, y: int, button: int) -> None:
        pass

    def on_mouse_leave(self, x: int, y: int) -> None:
        pass

    def on_mouse_leave_(self, x: int, y: int) -> None:
        self.on_mouse_leave(x, y)

    def on_global_mouse_move_(self, x: int, y: int) -> None:
        if self.is_main_window:
            self.on_global_mouse_move(x, y)
        if self.is_global_pressed and self.pressed_child:
            '''if not self.pressed_child == self.last_hover:
                if self.last_hover:
                    if self.last_hover.catch_mouse:
                        self.last_hover.on_mouse_leave_(x, y)
                    else:
                        self.last_hover.on_mouse_leave(x, y)
                self.last_hover = self.pressed_child'''
            if self.pressed_child.catch_mouse:
                return self.pressed_child.on_mouse_move_(x - self.scroll_x, y - self.scroll_y)
            else:
                return self.pressed_child.on_mouse_move(x - self.scroll_x, y - self.scroll_y)
        for child in self.child[::-1]:
            if not child.is_visible or not child.is_enabled\
                    or not is_object_colliding(child, x - self.scroll_x, y - self.scroll_y):
                continue
            if not child == self.last_hover:
                if self.last_hover:
                    if self.last_hover.catch_mouse:
                        self.last_hover.on_mouse_leave_(x, y)
                    else:
                        self.last_hover.on_mouse_leave(x, y)
                self.last_hover = child
            if child.catch_mouse:
                return child.on_mouse_move_(x - self.scroll_x, y - self.scroll_y)
            else:
                return child.on_mouse_move(x - self.scroll_x, y - self.scroll_y)
        if self.last_hover:
            if self.last_hover.catch_mouse:
                self.last_hover.on_mouse_leave_(x, y)
            else:
                self.last_hover.on_mouse_leave(x, y)
        self.last_hover = self
        return self.on_mouse_move(x, y)

    def on_global_mouse_down_(self, x: int, y: int, button: int) -> None:
        if self.is_main_window:
            self.is_global_pressed = True
            self.on_global_mouse_down(x, y, button)
        for child in self.child[::-1]:
            if not child.is_visible or not child.is_enabled\
                    or not is_object_colliding(child, x - self.scroll_x, y - self.scroll_y):
                continue
            child.is_pressed = True
            self.pressed_child = child
            child.last_btn = button
            if child.catch_mouse:
                return child.on_mouse_down_(x - self.scroll_x, y - self.scroll_y, button)
            else:
                return child.on_mouse_down(x - self.scroll_x, y - self.scroll_y, button)
        self.on_mouse_down(x, y, button)

    def on_global_mouse_up_(self, x: int, y: int, button: int) -> None:
        if self.is_main_window:
            self.on_global_mouse_up(x, y, button)
        self.is_global_pressed = False
        if not self.pressed_child:
            return
        self.pressed_child.is_pressed = False
        if self.pressed_child.catch_mouse:
            self.pressed_child.on_mouse_up_(x - self.scroll_x, y - self.scroll_y, button)
        else:
            self.pressed_child.on_mouse_up(x - self.scroll_x, y - self.scroll_y, button)
        self.pressed_child = None
        self.on_mouse_up(x, y, button)

    def on_global_mouse_leave(self, key_dict: dict) -> None:
        pass

    def on_global_mouse_enter(self, key_dict: dict) -> None:
        pass

    def on_key_down(self, key_dict: dict) -> None:
        pass

    def on_key_up(self, key_dict: dict) -> None:
        pass

    def on_focus_leave(self, key_dict: dict) -> None:
        pass

    def on_focus_enter(self, key_dict: dict) -> None:
        pass

    def on_key_down_(self, key_dict: dict) -> None:
        for child in self.child:
            if child.catch_keyboard and child.on_key_down_(key_dict):
                return
        self.on_key_down(key_dict)

    def on_key_up_(self, key_dict: dict) -> None:
        for child in self.child:
            if child.catch_keyboard and child.on_key_down_(key_dict):
                return
        self.on_key_up(key_dict)

    def add_child(self, child: any) -> None:
        self.child.append(child)
        self.child.sort(key=lambda x: x.z_order)

    def remove_child(self, child: any) -> None:
        self.child.remove(child)
        self.child.sort(key=lambda x: x.z_order)

    def draw(self, delta: int, parent: any = None):
        self.on_before_update()
        self.surface.fill(self.bg_color)
        if parent:
            parent.surface.blit(self.surface, (self.x, self.y))
        for child in self.child:
            if not child.is_visible:
                continue
            child.draw(delta, parent=self)
        self.on_after_update()

    def process_events(self, events: list) -> None:
        if not self.is_main_window:
            return
        for event in events:
            if event.type == pygame.VIDEORESIZE:
                self.w, self.h = self.surface.get_size()
                self.on_resize(self.w, self.h)
                continue
            if event.type == pygame.MOUSEMOTION:
                x, y = pygame.mouse.get_pos()
                self.on_global_mouse_move_(x, y)
                continue
            if event.type == pygame.MOUSEBUTTONDOWN:
                x, y = pygame.mouse.get_pos()
                self.on_global_mouse_down_(x, y, event.button)
                continue
            if event.type == pygame.MOUSEBUTTONUP:
                x, y = pygame.mouse.get_pos()
                self.on_global_mouse_up_(x, y, event.button)
                continue
            if event.type == pygame.KEYDOWN:
                self.on_key_down_(event.dict)
                continue
            if event.type == pygame.KEYUP:
                self.on_key_up_(event.dict)
                continue
            if event.type == pygame.QUIT:
                self.on_quit()
                continue
            if event.type == pygame.WINDOWLEAVE:
                self.on_global_mouse_leave(event.dict)
                continue
            if event.type == pygame.WINDOWENTER:
                self.on_global_mouse_enter(event.dict)
                continue
            if event.type == pygame.WINDOWFOCUSLOST:
                self.on_focus_leave(event.dict)
                continue
            if event.type == pygame.WINDOWFOCUSGAINED:
                self.on_focus_enter(event.dict)
                continue

    def resize(self, w: int, h: int) -> pygame.Surface:
        if self.is_main_window:
            self.surface = pygame.display.set_mode(
                (w, h),
                self.surface.get_flags(),
                self.depth, self.display, self.vsync
            )
        else:
            self.surface = pygame.transform.scale(self.surface, (w, h))
        self.w, self.h = self.surface.get_size()
        return self.surface

    def to_pillow_image(self) -> Image:
        return from_image(self.surface)



import os
import sys


msg_env_ = 'PYGAME_HIDE_SUPPORT_PROMPT'
show_msg_ = not bool(os.getenv(msg_env_))

if sys.modules.get('pygame'):
    if show_msg_:
        print('Using GSDL GUI. https://www.github.com/Pixelsuft/gsdl')
    pygame = sys.modules.get('pygame')
elif show_msg_:
    os.environ[msg_env_] = 'True'
    import pygame
    print('pygame {} (SDL {}.{}.{}, Integrated GSDL GUI, Python {}.{}.{})'.format(
        pygame.ver, *pygame.get_sdl_version() + sys.version_info[0:3]
    ))
    print('Hello from the pygame community. https://www.pygame.org/contribute.html')

del os, sys, pygame, msg_env_, show_msg_



import time


class GFps:
    def __init__(
        self, fps: any, smooth_fix: bool = False, no_limit: bool = False
    ) -> None:
        super(GFps, self).__init__()
        self.fps = float(fps)
        self.frame_rate = 1 / fps
        self.frame_rate_big = 1000 / fps
        self.smooth_fix = smooth_fix
        self.no_limit = no_limit
        self.last_tick = time.time()
        self.delta = round(1000 / fps)
        self.ideal_delta = round(1000 / fps)
        self.round_count = 2
        self.round_num = 10 ** self.round_count

    def try_tick(self) -> bool:
        now = time.time()
        if self.no_limit or now - self.last_tick >= self.frame_rate:
            self.delta = round((now - self.last_tick) * 1000) if not self.smooth_fix else self.frame_rate_big
            self.last_tick = now
            return True
        return False

    def get_fps(self) -> float:
        try:
            return 1000 / self.delta
        except ZeroDivisionError:
            return self.fps

    def get_int_fps(self) -> int:
        return round(self.get_fps())

    def set_round_num(self, round_num: int = 2) -> None:
        self.round_count = round_num
        self.round_num = 10 ** round_num

    def round_fps(self) -> str:
        fps_ = self.get_fps()
        return str(round(fps_ * self.round_num) / self.round_num)[:len((str(fps_))) + self.round_count + 1]



from .utils import *
from .frame import GFrame
import pygame


class GLabel:
    def __init__(
            self,
            parent: Union[GFrame, None],
            text: str,
            font: Union[pygame.font.Font, str],
            font_size: int = 0,
            color: tuple = (255, 255, 255),
            bg_color: tuple = None,
            x: int = 0,
            y: int = 0,
            w: int = 0,
            h: int = 0,
            alpha: bool = True,
            auto_size: bool = True,
            stretch: bool = False,
            alpha_blend: int = 255,
            anti_aliasing: bool = True,
            z_order: int = 0
    ):
        super(GLabel, self).__init__()
        self.parent = parent
        self.surface = parent.surface if parent else pygame.Surface((1, 1))
        self.x = x
        self.y = y
        self.alpha = alpha
        self.w, self.h = w, h
        self.text = text
        self.bg_color = bg_color
        self.color = color
        self.orig_image = self.surface
        self.image = self.orig_image
        self.auto_size = auto_size
        self.anti_aliasing = anti_aliasing
        self.stretch = stretch
        self.is_visible = True
        self.is_pressed = False
        self.is_enabled = True
        self.last_btn = 0
        self.catch_mouse = True
        self.catch_keyboard = False
        self.is_colliding = False
        self.z_order = z_order
        self.alpha_blend = alpha_blend
        self.font_size = font_size
        self.font = self.to_font(font)
        self.update_text()
        if not self.w or not self.h:
            self.w, self.h = self.image.get_size()
        if stretch:
            self.resize(self.w, self.h)
        if parent:
            self.parent.add_child(self)

    def set_text(self, new_text: str):
        self.text = new_text
        self.update_text()

    def update_text(self) -> None:
        self.orig_image = self.font.render(
            self.text, self.anti_aliasing, self.color, self.bg_color
        )
        if self.stretch:
            self.image = pygame.transform.scale(self.image, (self.w, self.h))
        else:
            self.image = self.orig_image
        if self.auto_size:
            self.w, self.h = self.image.get_size()
        self.image.set_alpha(self.alpha_blend)

    def to_font(self, font) -> pygame.font.Font:
        if type(font) == str:
            return pygame.font.Font(font, self.font_size)
        return font

    def resize(self, w: int, h: int) -> None:
        if self.auto_size:
            return
        self.w = w
        self.h = h
        if self.stretch:
            self.image = pygame.transform.scale(self.orig_image, (w, h))

    def draw(self, delta: float, parent: GFrame) -> None:
        if not self.parent:
            if not parent:
                return
            self.parent = parent
        if self.auto_size:
            self.surface.blit(
                self.image,
                (self.x + self.parent.scroll_x, self.y + self.parent.scroll_y)
            )
        else:
            self.surface.blit(
                self.image,
                (self.x + self.parent.scroll_x, self.y + self.parent.scroll_y),
                (self.x, self.y, self.w, self.h)
            )

    def to_pil_image(self) -> Image.Image:
        return from_image(self.image)

    def set_alpha(self, alpha: int):
        self.alpha_blend = alpha
        self.image.set_alpha(alpha)

    def on_mouse_move(self, x: int, y: int) -> None:
        pass

    def on_mouse_down(self, x: int, y: int, button: int = 0) -> None:
        pass

    def on_mouse_up(self, x: int, y: int, button: int = 0) -> None:
        pass

    def on_mouse_leave(self, x: int, y: int) -> None:
        pass

    def on_mouse_enter(self):
        pass

    def on_click(self, x, y):
        pass

    def on_mouse_move_(self, x: int, y: int) -> None:
        if self.is_pressed and self.last_btn == 1:
            if not is_object_colliding(self, x, y):
                self.is_colliding = False
                return self.on_mouse_move(x, y)
        if not self.is_colliding:
            self.is_colliding = True
            self.on_mouse_enter()
        self.on_mouse_move(x, y)

    def on_mouse_leave_(self, x: int, y: int) -> None:
        self.is_colliding = False
        self.on_mouse_leave(x, y)

    def on_mouse_down_(self, x: int, y: int, button: int = 0) -> None:
        self.on_mouse_down(x, y, button)

    def on_mouse_up_(self, x: int, y: int, button: int = 0) -> None:
        self.on_mouse_up(x, y, button)
        if self.last_btn == button == 1 and self.is_colliding:
            self.on_click(x, y)




from .utils import *
from .frame import GFrame
from io import BytesIO
from _io import BufferedReader
import pygame


class GImage:
    def __init__(
            self,
            parent: GFrame,
            image: Union[pygame.Surface, Image.Image, BytesIO, BufferedReader, str, bytes],
            x: int = 0,
            y: int = 0,
            w: int = 0,
            h: int = 0,
            alpha: bool = True,
            auto_size: bool = True,
            stretch: bool = False,
            alpha_blend: int = 255,
            z_order: int = 0
    ):
        super(GImage, self).__init__()
        self.parent = parent
        self.surface = parent.surface
        self.x = x
        self.y = y
        self.alpha = alpha
        self.w, self.h = w, h
        self.orig_image = self.surface
        self.image = self.orig_image
        self.auto_size = auto_size
        self.stretch = stretch
        self.is_visible = True
        self.catch_mouse = True
        self.is_colliding = False
        self.is_enabled = True
        self.is_pressed = False
        self.last_btn = 0
        self.catch_keyboard = False
        self.alpha_blend = alpha_blend
        self.update_image(image)
        self.z_order = z_order
        if not self.w or not self.h:
            self.w, self.h = self.image.get_size()
        if not alpha_blend == 255:
            self.set_alpha_blend(alpha_blend)
        if stretch:
            self.resize(self.w, self.h)
        self.parent.add_child(self)

    def to_img(self, image: any) -> pygame.Surface:
        tp = type(image)
        if tp == pygame.Surface:
            return image
        if tp == Image.Image:
            return to_image(image, alpha=self.alpha)
        if tp == str and file_exists(image):
            return pygame.image.load(image)
        if 'read' in dir(tp):
            return to_image(Image.open(tp), alpha=self.alpha)
        raise RuntimeError('Can\'t load image: ' + str(tp))

    def update_image(self, image: Union[pygame.Surface, Image.Image, BytesIO, BufferedReader, str, bytes]):
        if self.alpha:
            self.orig_image = self.to_img(image).convert_alpha()
        else:
            self.orig_image = self.to_img(image).convert()
        self.image = self.orig_image
        if self.auto_size:
            self.w, self.h = self.image.get_size()
        else:
            self.w, self.h = self.w, self.h
        self.image.set_alpha(self.alpha_blend)

    def resize(self, w: int, h: int) -> None:
        if self.auto_size:
            return
        self.w = w
        self.h = h
        if self.stretch:
            self.image = pygame.transform.scale(self.orig_image, (w, h))

    def to_pil_image(self) -> Image.Image:
        return from_image(self.image)

    def draw(self, delta: float, parent: GFrame) -> None:
        self.parent.surface.blit(
            self.image,
            (self.x + self.parent.scroll_x, self.y + self.parent.scroll_y),
            None if self.stretch else (0, 0, self.w, self.h)
        )

    def set_alpha_blend(self, alpha: int):
        self.alpha_blend = alpha
        self.image.set_alpha(alpha)

    def on_mouse_move(self, x: int, y: int) -> None:
        pass

    def on_mouse_down(self, x: int, y: int, button: int = 0) -> None:
        pass

    def on_mouse_up(self, x: int, y: int, button: int = 0) -> None:
        pass

    def on_mouse_leave(self, x: int, y: int) -> None:
        pass

    def on_mouse_enter(self):
        pass

    def on_click(self, x, y):
        pass

    def on_mouse_move_(self, x: int, y: int) -> None:
        if self.is_pressed and self.last_btn == 1:
            if not is_object_colliding(self, x, y):
                self.is_colliding = False
                return self.on_mouse_move(x, y)
        if not self.is_colliding:
            self.is_colliding = True
            self.on_mouse_enter()
        self.on_mouse_move(x, y)

    def on_mouse_leave_(self, x: int, y: int) -> None:
        self.is_colliding = False
        self.on_mouse_leave(x, y)

    def on_mouse_down_(self, x: int, y: int, button: int = 0) -> None:
        self.on_mouse_down(x, y, button)

    def on_mouse_up_(self, x: int, y: int, button: int = 0) -> None:
        self.on_mouse_up(x, y, button)
        if self.last_btn == button == 1 and self.is_colliding:
            self.on_click(x, y)



import os
import sys
import json
import ctypes
import pygame
from typing import Optional, Tuple, List, Union, IO, Literal
from PIL import Image
try:
    import android
    is_android = True
    from android.permissions import request_permissions, Permission
except ImportError:
    is_android = False


encoding_ = sys.getdefaultencoding()
is_win = 'windll' in dir(ctypes)
force_fullscreen = is_android
force_fullscreen_flag = pygame.FULLSCREEN if force_fullscreen else 0
cur_path = os.getcwd()


def request_storage_permissions() -> None:
    if not is_android:
        return
    request_permissions([
        Permission.READ_EXTERNAL_STORAGE,
        Permission.WRITE_EXTERNAL_STORAGE
    ])


def p(*path) -> str:
    return os.path.join(cur_path, *path)


def to_image(pillow_img: Image.Image, alpha=True) -> pygame.Surface:
    result = pygame.image.fromstring(
        pillow_img.tobytes(), pillow_img.size, pillow_img.mode
    )
    return result.convert_alpha() if alpha else result.convert()


def from_image(pygame_image: pygame.Surface, mode: str = 'RGBA', is_flipped: bool = False) -> Image.Image:
    return Image.frombytes(
        mode,
        pygame_image.get_size(),
        pygame.image.tostring(
            pygame_image,
            mode,
            is_flipped
        ),
    )


def file_exists(filename: str) -> bool:
    return os.access(filename, os.F_OK)


def fast_read(filename: str, response_type: type = str, encoding: str = 'utf-8') -> Union[str, bytes]:
    temp_file = open(filename, 'rb')
    result = temp_file.read()
    temp_file.close()
    return result if not response_type == str else result.decode(encoding)


def fast_write(filename: str, content) -> int:
    temp_file = open(filename, 'w' if type(content) == str else 'wb')
    result = temp_file.write(content)
    temp_file.close()
    return result


def read_json(filename: str) -> dict:
    return json.loads(fast_read(filename))


def write_json(filename: str, json_: dict) -> int:
    return fast_write(filename, json.dumps(json_))


def is_object_colliding(object_: any, x: int, y: int) -> bool:
    return object_.x + object_.w > x > object_.x and object_.y + object_.h > y > object_.y



import os
import sys
import ctypes
from urllib.request import urlretrieve as download
from setuptools import setup, find_packages


try:
    from pip._internal.cli.main import main as pip_main
except ImportError:
    raise RuntimeError('Please, install pip before running gsdl installation')


is_win = sys.platform.lower().strip() == 'win32'
backup = (sys.argv, sys.exit)
requirements = [
    'pygame',
    'Pillow',
    'buildozer'
]
readme = ''
license_ = ''
version = '0.0.0'
to_remove = []


for i in range(len(sys.argv)):
    if sys.argv[i].lower().strip() == '-v' and len(sys.argv) > i + 1:
        version = sys.argv[i + 1].lower().strip()
        to_remove.append(sys.argv[i])
        to_remove.append(sys.argv[i + 1])
for i in tuple(to_remove):
    sys.argv.remove(i)


if os.access('LICENSE', os.F_OK):
    temp_file = open('LICENSE', 'r')
    license_ = temp_file.read()
    temp_file.close()
else:
    download('https://github.com/Pixelsuft/gsdl/raw/main/LICENSE', 'LICENSE')
    temp_file = open('LICENSE', 'r')
    license_ = temp_file.read()
    temp_file.close()
    os.remove('LICENSE')


if os.access('README.MD', os.F_OK):
    temp_file = open('README.MD', 'r')
    readme = temp_file.read()
    temp_file.close()
else:
    download('https://github.com/Pixelsuft/gsdl/raw/main/README.md', 'README.MD')
    temp_file = open('README.MD', 'r')
    readme = temp_file.read()
    temp_file.close()
    os.remove('README.MD')


def fake_exit(*args, **kwargs) -> None:
    pass


def hook_sys(args: tuple) -> None:
    sys.argv = list(args)
    sys.exit = fake_exit


def unhook_sys() -> None:
    sys.argv = backup[0]
    sys.exit = backup[1]


def parse_args(cmd_string: str) -> list:
    result_str = ''
    work = str(cmd_string).strip()
    can_pass_space = False
    for i in range(len(work)):
        if work[i] == '\"':
            if not can_pass_space:
                can_pass_space = True
                result_str += '\n'
            else:
                can_pass_space = False
        elif work[i] == ' ':
            if not can_pass_space:
                result_str += '\n'
            else:
                result_str += ' '
        else:
            result_str += work[i]
    result = []
    for i in result_str.split('\n'):
        if not i == '':
            result.append(i)
    return result


def check_cmd(cmd: list) -> list:
    result = cmd
    return result


def run_pip_command(cmd) -> any:
    if not cmd:
        return True
    if type(cmd) == str:
        cmd = parse_args(cmd)
    cmd = tuple(check_cmd(list(cmd)))
    hook_sys(cmd)
    error = None
    try:
        pip_main()
    except Exception as e:
        error = e
    unhook_sys()
    return error if error else False


for i in license_.split('\n'):
    spaces = int(round(os.get_terminal_size()[0] / 2 - len(i) / 2))
    print(' ' * spaces + i)

print('\n\n', end='')

for i in requirements:
    if not i.strip().lower():
        continue
    print(f'Installing {i}...')
    run_pip_command(f'pip install "{i}"')


setup(
    name="gsdl",
    author="Pixelsuft",
    url="https://github.com/Pixelsuft/gsdl",
    project_urls={
        "Readme": "https://github.com/Pixelsuft/gsdl/blob/main/README.MD",
        "Example": "https://github.com/Pixelsuft/gsdl/blob/main/main.py",
        "Issue tracker": "https://github.com/Pixelsuft/gsdl/issues",
    },
    version=version,
    packages=find_packages(),
    license="MIT",
    description="Pygame GUI Library.",
    long_description=readme,
    long_description_content_type="text/markdown",
    include_package_data=True,
    install_requires=requirements,
    python_requires=">=3.6",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    zip_safe=False,
    py_modules=["gsdl"],
    package_dir={'': '.'},
    entry_points={"console_scripts": ["gsdl = gsdl.__main__:main"]},
    keywords="gsdl"
)



from . import hellomsg
from .fps_clock import GFps
from .utils import *
from .frame import GFrame
from .image import GImage
from .label import GLabel
from .rect import GRect
from .simple_button import GSimpleButton
from .win_button import GWinButton

del hellomsg
